use std::fs;
use std::process::Command;
use std::process::Stdio;

use super::Executor;

use super::helper::*;

#[derive(Debug, Clone, Copy)]
pub struct PhpExecutor {}

impl Executor for PhpExecutor {
    fn id(&self) -> String {
        "php".into()
    }

    fn exec(&self, code: String) -> Result<String, String> {
        assure_dir_exists().map_err(|x| x.to_string())?;

        let path = create_path();

        // 1. Save the code
        fs::write(path.clone(), code).map_err(|x| x.to_string())?;

        // 2. Run it!
        let mut out = Command::new("php")
            .arg(path)
            .stdout(Stdio::piped())
            .spawn()
            .map_err(|x| x.to_string())?;

        // 3. Grab the exit code
        let exit_status = out.wait().map_err(|x| x.to_string())?;

        let mut stdout: String = get_stdout(out.stdout)?;

        stdout += "\n\n";
        stdout += exit_code_msg(exit_status).as_str();

        Ok(stdout)
    }
}